globalThis.__RSC_MANIFEST=(globalThis.__RSC_MANIFEST||{});globalThis.__RSC_MANIFEST["/api/runs/route"]={"moduleLoading":{"prefix":"/_next/","crossOrigin":null},"ssrModuleMapping":{"16585":{"*":{"id":"65329","name":"*","chunks":[],"async":false}},"19638":{"*":{"id":"79563","name":"*","chunks":[],"async":false}},"23726":{"*":{"id":"62302","name":"*","chunks":[],"async":false}},"39158":{"*":{"id":"14407","name":"*","chunks":[],"async":false}},"39275":{"*":{"id":"9587","name":"*","chunks":[],"async":false}},"53120":{"*":{"id":"62118","name":"*","chunks":[],"async":false}},"61343":{"*":{"id":"50599","name":"*","chunks":[],"async":false}},"66513":{"*":{"id":"52144","name":"*","chunks":[],"async":false}},"76130":{"*":{"id":"63582","name":"*","chunks":[],"async":false}}},"edgeSSRModuleMapping":{},"clientModules":{"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/client-page.js":{"id":66513,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/client-page.js":{"id":66513,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/client-segment.js":{"id":23726,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/client-segment.js":{"id":23726,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/error-boundary.js":{"id":76130,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/error-boundary.js":{"id":76130,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/layout-router.js":{"id":39275,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/layout-router.js":{"id":39275,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/not-found-boundary.js":{"id":16585,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/not-found-boundary.js":{"id":16585,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/client/components/render-from-template-context.js":{"id":61343,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/client/components/render-from-template-context.js":{"id":61343,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/lib/metadata/metadata-boundary.js":{"id":53120,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/node_modules/next/dist/esm/lib/metadata/metadata-boundary.js":{"id":53120,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/globals.css":{"id":45214,"name":"*","chunks":["185","static/chunks/app/layout-ae9ba5f4cc8e2f29.js"],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/components/dataset-viewer/DatasetViewer.tsx":{"id":19638,"name":"*","chunks":[],"async":false},"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/page.tsx":{"id":39158,"name":"*","chunks":["547","static/chunks/547-ef39df789a1cd585.js","883","static/chunks/883-1b43147b873c06b7.js","931","static/chunks/app/page-52342a85c0f0c116.js"],"async":false}},"entryCSSFiles":{"/home/charlieji/workspace/bella/bespoke-dataset-viewer/":[],"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/layout":["static/css/2a4e445879690d16.css"],"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/page":[],"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/api/runs/route":[]},"rscModuleMapping":{"16585":{"*":{"id":"56613","name":"*","chunks":[],"async":false}},"19638":{"*":{"id":"34727","name":"*","chunks":[],"async":false}},"23726":{"*":{"id":"42461","name":"*","chunks":[],"async":false}},"39158":{"*":{"id":"23068","name":"*","chunks":[],"async":false}},"39275":{"*":{"id":"15282","name":"*","chunks":[],"async":false}},"45214":{"*":{"id":"67272","name":"*","chunks":[],"async":false}},"53120":{"*":{"id":"58825","name":"*","chunks":[],"async":false}},"61343":{"*":{"id":"5486","name":"*","chunks":[],"async":false}},"66513":{"*":{"id":"46472","name":"*","chunks":[],"async":false}},"76130":{"*":{"id":"48190","name":"*","chunks":[],"async":false}}},"edgeRscModuleMapping":{}}