(()=>{var e={};e.id=409,e.ids=[409],e.modules={20399:e=>{"use strict";e.exports=require("next/dist/compiled/next-server/app-page.runtime.prod.js")},209:e=>{"use strict";e.exports=require("next/dist/server/app-render/action-async-storage.external.js")},79348:e=>{"use strict";e.exports=require("next/dist/server/app-render/work-async-storage.external.js")},30412:e=>{"use strict";e.exports=require("next/dist/server/app-render/work-unit-async-storage.external.js")},55315:e=>{"use strict";e.exports=require("path")},59073:(e,r,t)=>{"use strict";t.r(r),t.d(r,{GlobalError:()=>i.a,__next_app__:()=>u,pages:()=>p,routeModule:()=>m,tree:()=>l});var n=t(49442),s=t(10042),o=t(48190),i=t.n(o),a=t(63289),d={};for(let e in a)0>["default","tree","pages","GlobalError","__next_app__","routeModule"].indexOf(e)&&(d[e]=()=>a[e]);t.d(r,d);let l=["",{children:["/_not-found",{children:["__PAGE__",{},{page:[()=>Promise.resolve().then(t.t.bind(t,6042,23)),"next/dist/client/components/not-found-error"]}]},{}]},{layout:[()=>Promise.resolve().then(t.bind(t,44486)),"/home/charlieji/workspace/bella/bespoke-dataset-viewer/app/layout.tsx"],"not-found":[()=>Promise.resolve().then(t.t.bind(t,6042,23)),"next/dist/client/components/not-found-error"]}],p=[],u={require:t,loadChunk:()=>Promise.resolve()},m=new n.AppPageRouteModule({definition:{kind:s.RouteKind.APP_PAGE,page:"/_not-found/page",pathname:"/_not-found",bundlePath:"",filename:"",appPaths:[]},userland:{loaderTree:l}})},26699:()=>{},49826:()=>{},22119:(e,r,t)=>{Promise.resolve().then(t.t.bind(t,46472,23)),Promise.resolve().then(t.t.bind(t,42461,23)),Promise.resolve().then(t.t.bind(t,48190,23)),Promise.resolve().then(t.t.bind(t,15282,23)),Promise.resolve().then(t.t.bind(t,56613,23)),Promise.resolve().then(t.t.bind(t,5486,23)),Promise.resolve().then(t.t.bind(t,58825,23))},74617:(e,r,t)=>{Promise.resolve().then(t.t.bind(t,52144,23)),Promise.resolve().then(t.t.bind(t,62302,23)),Promise.resolve().then(t.t.bind(t,63582,23)),Promise.resolve().then(t.t.bind(t,9587,23)),Promise.resolve().then(t.t.bind(t,65329,23)),Promise.resolve().then(t.t.bind(t,50599,23)),Promise.resolve().then(t.t.bind(t,62118,23))},44486:(e,r,t)=>{"use strict";t.r(r),t.d(r,{default:()=>o,metadata:()=>s});var n=t(47705);t(67272);let s={title:"Bella Dataset Viewer",description:"A powerful dataset viewer and analysis tool"};function o({children:e}){return(0,n.jsx)("html",{lang:"en",suppressHydrationWarning:!0,children:(0,n.jsx)("body",{suppressHydrationWarning:!0,children:e})})}},67272:()=>{}};var r=require("../../webpack-runtime.js");r.C(e);var t=e=>r(r.s=e),n=r.X(0,[492,146],()=>t(59073));module.exports=n})();